@@ -5,9 +5,9 @@ from __future__ import unicode_literals, print_function
 import os
 
 import pytest
+import inspect
 
-import xonsh.built_ins as built_ins
-from xonsh.aliases import Aliases
+from xonsh.aliases import Aliases, ExecAlias
 from xonsh.environ import Env
 
 from tools import skip_if_on_windows
@@ -57,9 +57,9 @@ def test_eval_recursive(xonsh_execer, xonsh_builtins):
 
 
 @skip_if_on_windows
-def test_eval_recursive_callable_partial(xonsh_execer, xonsh_builtins):
+def test_eval_recursive_callable_partial(xonsh_execer, xession):
     ales = make_aliases()
-    xonsh_builtins.__xonsh__.env = Env(HOME=os.path.expanduser("~"))
+    xession.env = Env(HOME=os.path.expanduser("~"))
     assert ales.get("indirect_cd")(["arg2", "arg3"]) == ["..", "arg2", "arg3"]
 
 
@@ -113,3 +113,82 @@ def test_recursive_callable_partial_none(xonsh_execer, xonsh_builtins):
     args, obs = alias()
     assert args == "wakka"
     assert len(obs) == 0
+
+
+@pytest.mark.parametrize(
+    "alias",
+    [
+        "echo 'hi' and echo 'there'",
+        "echo 'hi' or echo 'there'",
+        "echo 'hi' && echo 'there'",
+        "echo 'hi' || echo 'there'",
+        "echo 'hi';  echo 'there'",
+    ],
+)
+def test_subprocess_logical_operators(xonsh_execer, xonsh_builtins, alias):
+    ales = make_aliases()
+    ales["echocat"] = alias
+    assert isinstance(ales["echocat"], ExecAlias)
+
+
+@pytest.mark.parametrize(
+    "alias",
+    [
+        "echo 'hi' | grep h",
+        "echo 'hi' > file",
+        "cat < file",
+        "COMMAND1 e>o < input.txt | COMMAND2 > output.txt e>> errors.txt",
+        "echo 'h|i' | grep h",
+        "echo 'h|i << x > 3' | grep x",
+    ],
+)
+def test_subprocess_io_operators(xonsh_execer, xonsh_builtins, alias):
+    ales = make_aliases()
+    ales["echocat"] = alias
+    assert isinstance(ales["echocat"], ExecAlias)
+
+
+@pytest.mark.parametrize(
+    "alias",
+    [
+        {"echocat": "ls"},
+    ],
+)
+def test_dict_merging(xonsh_execer, xonsh_builtins, alias):
+    ales = make_aliases()
+    assert (ales | alias)["echocat"] == ["ls"]
+    assert (alias | ales)["echocat"] == ["ls"]
+    assert "echocat" not in ales
+
+
+@pytest.mark.parametrize(
+    "alias",
+    [
+        {"echocat": "echo Why do people still use python2.7?"},
+        {"echocat": "echo Why?"},
+    ],
+)
+def test_dict_merging_assignment(xonsh_execer, xonsh_builtins, alias):
+    ales = make_aliases()
+    ales |= alias
+
+    assert "echocat" in ales
+    assert " ".join(ales["echocat"]) == alias["echocat"]
+
+    ales = make_aliases()
+    alias |= ales
+
+    assert "o" in alias
+    assert alias["o"] == ales["o"]
+
+
+def test_exec_alias_args(xonsh_execer, xonsh_builtins):
+    stack = inspect.stack()
+    try:
+        ExecAlias("myargs = $args")(["arg0"], stack=stack)
+        ExecAlias("myarg0 = $arg0")(["arg0"], stack=stack)
+    except KeyError:
+        assert False
+
+    assert stack[0][0].f_locals["myargs"] == ["arg0"]
+    assert stack[0][0].f_locals["myarg0"] == "arg0"
