@@ -1,5 +1,7 @@
+"""Tests involving running Xonsh in subproc.
+This requires Xonsh installed in venv or otherwise available on PATH
+"""
 import os
-import sys
 import shutil
 import tempfile
 import subprocess as sp
@@ -10,74 +12,75 @@ import xonsh
 from xonsh.lib.os import indir
 
 from tools import (
+    skip_if_on_msys,
     skip_if_on_windows,
+    skip_if_on_darwin,
     ON_WINDOWS,
     ON_DARWIN,
     ON_TRAVIS,
 )
 
 
-XONSH_PREFIX = xonsh.__file__
-if "site-packages" in XONSH_PREFIX:
-    # must be installed version of xonsh
-    num_up = 5
-else:
-    # must be in source dir
-    num_up = 2
-for i in range(num_up):
-    XONSH_PREFIX = os.path.dirname(XONSH_PREFIX)
 PATH = (
-    os.path.join(os.path.dirname(__file__), "bin")
-    + os.pathsep
-    + os.path.join(XONSH_PREFIX, "bin")
-    + os.pathsep
-    + os.path.join(XONSH_PREFIX, "Scripts")
-    + os.pathsep
-    + os.path.join(XONSH_PREFIX, "scripts")
-    + os.pathsep
-    + os.path.dirname(sys.executable)
+    os.path.join(os.path.abspath(os.path.dirname(__file__)), "bin")
     + os.pathsep
     + os.environ["PATH"]
 )
 
 
 skip_if_no_xonsh = pytest.mark.skipif(
-    shutil.which("xonsh", path=PATH) is None, reason="xonsh not on PATH"
+    shutil.which("xonsh") is None, reason="xonsh not on PATH"
 )
 skip_if_no_make = pytest.mark.skipif(
-    shutil.which("make", path=PATH) is None, reason="make command not on PATH"
+    shutil.which("make") is None, reason="make command not on PATH"
 )
 skip_if_no_sleep = pytest.mark.skipif(
-    shutil.which("sleep", path=PATH) is None, reason="sleep command not on PATH"
+    shutil.which("sleep") is None, reason="sleep command not on PATH"
 )
 
 
-def run_xonsh(cmd, stdin=sp.PIPE, stdout=sp.PIPE, stderr=sp.STDOUT):
+def run_xonsh(
+    cmd,
+    stdin=sp.PIPE,
+    stdout=sp.PIPE,
+    stderr=sp.STDOUT,
+    single_command=False,
+    interactive=False,
+):
     env = dict(os.environ)
     env["PATH"] = PATH
-    env["XONSH_DEBUG"] = "1"
+    env["XONSH_DEBUG"] = "0"  # was "1"
     env["XONSH_SHOW_TRACEBACK"] = "1"
     env["RAISE_SUBPROC_ERROR"] = "0"
     env["PROMPT"] = ""
-    xonsh = "xonsh.bat" if ON_WINDOWS else "xon.sh"
-    xonsh = shutil.which(xonsh, path=PATH)
+    xonsh = shutil.which("xonsh", path=PATH)
+    args = [xonsh, "--no-rc"]
+    if interactive:
+        args.append("-i")
+    if single_command:
+        args += ["-c", cmd]
+        input = None
+    else:
+        input = cmd
+
     proc = sp.Popen(
-        [xonsh, "--no-rc"],
+        args,
         env=env,
         stdin=stdin,
         stdout=stdout,
         stderr=stderr,
         universal_newlines=True,
     )
+
     try:
-        out, err = proc.communicate(input=cmd, timeout=10)
+        out, err = proc.communicate(input=input, timeout=20)
     except sp.TimeoutExpired:
         proc.kill()
         raise
     return out, err, proc.returncode
 
 
-def check_run_xonsh(cmd, fmt, exp):
+def check_run_xonsh(cmd, fmt, exp, exp_rtn=0):
     """The ``fmt`` parameter is a function
     that formats the output of cmd, can be None.
     """
@@ -87,7 +90,7 @@ def check_run_xonsh(cmd, fmt, exp):
     if callable(exp):
         exp = exp()
     assert out == exp, err
-    assert rtn == 0, err
+    assert rtn == exp_rtn, err
 
 
 #
@@ -175,6 +178,22 @@ def _test_stream(args, stdin, stdout, stderr):
 aliases['test-stream'] = _test_stream
 x = !(test-stream)
 print(x.returncode)
+""",
+        "1\n",
+        0,
+    ),
+    # test captured streaming alias without stderr
+    (
+        """
+def _test_stream(args, stdin, stdout, stderr):
+    print('hallo on err', file=stderr)
+    print('hallo on out', file=stdout)
+    return 1
+
+aliases['test-stream'] = _test_stream
+with __xonsh__.env.swap(XONSH_SUBPROC_CAPTURED_PRINT_STDERR=True):
+    x = !(test-stream)
+    print(x.returncode)
 """,
         "hallo on err\n1\n",
         0,
@@ -217,9 +236,9 @@ g
 with open('tttt', 'w') as fp:
     fp.write("Wow mom!\\n")
 
-![cat tttt | wc]
+![python tests/bin/cat tttt | python tests/bin/wc]
 """,
-        "      1       2      10\n" if ON_WINDOWS else " 1  2 9 <stdin>\n",
+        " 1  2 10 <stdin>\n" if ON_WINDOWS else " 1  2 9 <stdin>\n",
         0,
     ),
     # test double  piping 'real' command
@@ -228,9 +247,9 @@ with open('tttt', 'w') as fp:
 with open('tttt', 'w') as fp:
     fp.write("Wow mom!\\n")
 
-![cat tttt | wc | wc]
+![python tests/bin/cat tttt | python tests/bin/wc | python tests/bin/wc]
 """,
-        "      1       3      24\n" if ON_WINDOWS else " 1  4 16 <stdin>\n",
+        " 1  4 18 <stdin>\n" if ON_WINDOWS else " 1  4 16 <stdin>\n",
         0,
     ),
     # test unthreadable alias (which should trigger a ProcPoxy call)
@@ -289,10 +308,11 @@ def _echo(args):
     print(' '.join(args))
 aliases['echo'] = _echo
 
-echo --option1 \
+echo --option1 \\
 --option2
-""",
-        "--option1 --option2\n",
+echo missing \\
+EOL""",
+        "--option1 --option2\nmissing EOL\n",
         0,
     ),
     #
@@ -341,7 +361,7 @@ echo foo_@$(echo spam sausage)_bar
     (
         """
 echo Just the place for a snark. >tttt
-cat tttt
+python tests/bin/cat tttt
 """,
         "Just the place for a snark.\n",
         0,
@@ -415,14 +435,98 @@ else:
         "Var foo\n",
         0,
     ),
+    #
+    # test logical subprocess operators
+    #
+    (
+        """
+def _echo(args):
+    print(' '.join(args))
+aliases['echo'] = _echo
+
+echo --version and echo a
+echo --version && echo a
+echo --version or echo a
+echo --version || echo a
+echo -+version and echo a
+echo -+version && echo a
+echo -+version or echo a
+echo -+version || echo a
+echo -~version and echo a
+echo -~version && echo a
+echo -~version or echo a
+echo -~version || echo a
+""",
+        """--version
+a
+--version
+a
+--version
+--version
+-+version
+a
+-+version
+a
+-+version
+-+version
+-~version
+a
+-~version
+a
+-~version
+-~version
+""",
+        0,
+    ),
 ]
 
+UNIX_TESTS = [
+    # testing alias stack: lambda function
+    (
+        """
+def _echo():
+    echo hello
 
+aliases['echo'] = _echo
+echo
+""",
+        "hello\n",
+        0,
+    ),
+    # testing alias stack: ExecAlias
+    (
+        """
+aliases['echo'] = "echo @('hello')"
+echo
+""",
+        "hello\n",
+        0,
+    ),
+    # testing alias stack: callable alias (ExecAlias) + no binary location + infinite loop
+    (
+        """
+aliases['first'] = "second @(1)"
+aliases['second'] = "first @(1)"
+first
+""",
+        lambda out: 'Recursive calls to "first" alias.' in out,
+        0,
+    ),
+]
+
+if not ON_WINDOWS:
+    ALL_PLATFORMS = tuple(ALL_PLATFORMS) + tuple(UNIX_TESTS)
+
+
+@skip_if_no_xonsh
 @pytest.mark.parametrize("case", ALL_PLATFORMS)
 def test_script(case):
     script, exp_out, exp_rtn = case
     out, err, rtn = run_xonsh(script)
-    assert exp_out == out
+    if callable(exp_out):
+        assert exp_out(out)
+    else:
+        assert exp_out == out
     assert exp_rtn == rtn
 
 
@@ -442,6 +546,7 @@ f o>e
 ]
 
 
+@skip_if_no_xonsh
 @pytest.mark.parametrize("case", ALL_PLATFORMS_STDERR)
 def test_script_stderr(case):
     script, exp_err, exp_rtn = case
@@ -450,6 +555,7 @@ def test_script_stderr(case):
     assert exp_rtn == rtn
 
 
+@skip_if_no_xonsh
 @skip_if_on_windows
 @pytest.mark.parametrize(
     "cmd, fmt, exp",
@@ -457,12 +563,18 @@ def test_script_stderr(case):
         ("pwd", None, lambda: os.getcwd() + "\n"),
         ("echo WORKING", None, "WORKING\n"),
         ("ls -f", lambda out: out.splitlines().sort(), os.listdir().sort()),
+        (
+            "$FOO='foo' $BAR=2 xonsh -c r'echo -n $FOO$BAR'",
+            None,
+            "foo2",
+        ),
     ],
 )
 def test_single_command_no_windows(cmd, fmt, exp):
     check_run_xonsh(cmd, fmt, exp)
 
 
+@skip_if_no_xonsh
 def test_eof_syntax_error():
     """Ensures syntax errors for EOF appear on last line."""
     script = "x = 1\na = (1, 0\n"
@@ -471,6 +583,7 @@ def test_eof_syntax_error():
     assert ":2:0: EOF in multi-line statement" in err
 
 
+@skip_if_no_xonsh
 def test_open_quote_syntax_error():
     script = (
         "#!/usr/bin/env xonsh\n\n"
@@ -490,21 +603,45 @@ _bad_case = pytest.mark.skipif(
 )
 
 
+@skip_if_no_xonsh
+def test_atdollar_no_output():
+    # see issue 1521
+    script = """
+def _echo(args):
+    print(' '.join(args))
+aliases['echo'] = _echo
+@$(echo)
+"""
+    out, err, rtn = run_xonsh(script, stderr=sp.PIPE)
+    assert "command is empty" in err
+
+
+@skip_if_no_xonsh
+def test_empty_command():
+    script = "$['']\n"
+    out, err, rtn = run_xonsh(script, stderr=sp.PIPE)
+    assert "command is empty" in err
+
+
+@skip_if_no_xonsh
 @_bad_case
 def test_printfile():
     check_run_xonsh("printfile.xsh", None, "printfile.xsh\n")
 
 
+@skip_if_no_xonsh
 @_bad_case
 def test_printname():
     check_run_xonsh("printfile.xsh", None, "printfile.xsh\n")
 
 
+@skip_if_no_xonsh
 @_bad_case
 def test_sourcefile():
     check_run_xonsh("printfile.xsh", None, "printfile.xsh\n")
 
 
+@skip_if_no_xonsh
 @_bad_case
 @pytest.mark.parametrize(
     "cmd, fmt, exp",
@@ -537,6 +674,7 @@ def test_subshells(cmd, fmt, exp):
     check_run_xonsh(cmd, fmt, exp)
 
 
+@skip_if_no_xonsh
 @skip_if_on_windows
 @pytest.mark.parametrize("cmd, exp", [("pwd", lambda: os.getcwd() + "\n")])
 def test_redirect_out_to_file(cmd, exp, tmpdir):
@@ -553,7 +691,7 @@ def test_redirect_out_to_file(cmd, exp, tmpdir):
 @skip_if_no_xonsh
 @skip_if_no_sleep
 @skip_if_on_windows
-@pytest.mark.xfail(strict=False) # TODO: fixme (super flaky on OSX)
+@pytest.mark.xfail(strict=False)  # TODO: fixme (super flaky on OSX)
 def test_xonsh_no_close_fds():
     # see issue https://github.com/xonsh/xonsh/issues/2984
     makefile = (
@@ -574,14 +712,19 @@ def test_xonsh_no_close_fds():
         assert "warning" not in out
 
 
+@skip_if_no_xonsh
 @pytest.mark.parametrize(
-    "cmd, fmt, exp", [("ls | wc", lambda x: x > "", True),],  # noqa E231 (black removes space)  
+    "cmd, fmt, exp",
+    [
+        ("cat tttt | wc", lambda x: x > "", True),
+    ],  # noqa E231 (black removes space)
 )
 def test_pipe_between_subprocs(cmd, fmt, exp):
     "verify pipe between subprocesses doesn't throw an exception"
     check_run_xonsh(cmd, fmt, exp)
 
 
+@skip_if_no_xonsh
 @skip_if_on_windows
 def test_negative_exit_codes_fail():
     # see issue 3309
@@ -591,6 +734,7 @@ def test_negative_exit_codes_fail():
     assert "OK" != err
 
 
+@skip_if_no_xonsh
 @pytest.mark.parametrize(
     "cmd, exp",
     [
@@ -612,3 +756,60 @@ aliases['echo'] = _echo
     )
     out, _, _ = run_xonsh(script)
     assert out == exp
+
+
+# issue 3402
+@skip_if_no_xonsh
+@skip_if_on_windows
+@pytest.mark.parametrize(
+    "cmd, exp_rtn",
+    [
+        ("import sys; sys.exit(0)", 0),
+        ("import sys; sys.exit(100)", 100),
+        ("sh -c 'exit 0'", 0),
+        ("sh -c 'exit 1'", 1),
+    ],
+)
+def test_single_command_return_code(cmd, exp_rtn):
+    _, _, rtn = run_xonsh(cmd, single_command=True)
+    assert rtn == exp_rtn
+
+
+@skip_if_no_xonsh
+@skip_if_on_msys
+@skip_if_on_windows
+@skip_if_on_darwin
+def test_argv0():
+    check_run_xonsh("checkargv0.xsh", None, "OK\n")
+
+
+@pytest.mark.parametrize("interactive", [True, False])
+def test_loading_correctly(monkeypatch, interactive):
+    # Ensure everything loads correctly in interactive mode (e.g. #4289)
+    monkeypatch.setenv("SHELL_TYPE", "prompt_toolkit")
+    monkeypatch.setenv("XONSH_LOGIN", "1")
+    monkeypatch.setenv("XONSH_INTERACTIVE", "1")
+    out, err, ret = run_xonsh(
+        "import xonsh; echo -n AAA @(xonsh.__file__) BBB",
+        interactive=interactive,
+        single_command=True,
+    )
+    assert not err
+    assert ret == 0
+    our_xonsh = (
+        xonsh.__file__
+    )  # make sure xonsh didn't fail and fallback to the system shell
+    assert f"AAA {our_xonsh} BBB" in out  # ignore tty warnings/prompt text
+
+
+@skip_if_no_xonsh
+@pytest.mark.parametrize(
+    "cmd",
+    [
+        "x = 0; (lambda: x)()",
+        "x = 0; [x for _ in [0]]",
+    ],
+)
+def test_exec_function_scope(cmd):
+    _, _, rtn = run_xonsh(cmd, single_command=True)
+    assert rtn == 0
