@@ -10,10 +10,7 @@ from pprint import pformat
 
 import pytest
 
-try:
-    from ply.lex import LexToken
-except ImportError:
-    from xonsh.ply.ply.lex import LexToken
+from xonsh.ply.ply.lex import LexToken
 
 
 from xonsh.lexer import Lexer
@@ -463,3 +460,25 @@ def test_lexer_split(s, exp):
     lexer = Lexer()
     obs = lexer.split(s)
     assert exp == obs
+
+
+@pytest.mark.parametrize(
+    "s",
+    (
+        "()",  # sanity
+        "(",
+        ")",
+        "))",
+        "'string\nliteral",
+        "'''string\nliteral",
+        "string\nliteral'",
+        '"',
+        "'",
+        '"""',
+    ),
+)
+def test_tolerant_lexer(s):
+    lexer = Lexer(tolerant=True)
+    lexer.input(s)
+    error_tokens = list(tok for tok in lexer if tok.type == "ERRORTOKEN")
+    assert all(tok.value in s for tok in error_tokens)  # no error messages
