@@ -1,17 +1,17 @@
 # -*- coding: utf-8 -*-
 """Tests the xonsh parser."""
 import ast
-import builtins
 import textwrap
 import itertools
 
 import pytest
 
 from xonsh.ast import AST, With, Pass, Str, Call
+from xonsh.built_ins import XSH
 from xonsh.parser import Parser
 from xonsh.parsers.fstring_adaptor import FStringAdaptor
 
-from tools import nodes_equal, skip_if_no_walrus, VER_MAJOR_MINOR
+from tools import nodes_equal, skip_if_pre_3_8, VER_MAJOR_MINOR
 
 
 @pytest.fixture(autouse=True)
@@ -19,7 +19,7 @@ def xonsh_builtins_autouse(xonsh_builtins):
     return xonsh_builtins
 
 
-PARSER = Parser(lexer_optimize=False, yacc_optimize=False, yacc_debug=True)
+PARSER = Parser(yacc_optimize=False, yacc_debug=True)
 
 
 def check_ast(inp, run=True, mode="eval", debug_level=0):
@@ -43,8 +43,7 @@ def check_stmts(inp, run=True, mode="exec", debug_level=0):
 
 
 def check_xonsh_ast(xenv, inp, run=True, mode="eval", debug_level=0, return_obs=False):
-    __tracebackhide__ = True
-    builtins.__xonsh__.env = xenv
+    XSH.env = xenv
     obs = PARSER.parse(inp, debug_level=debug_level)
     if obs is None:
         return  # comment only
@@ -61,6 +60,12 @@ def check_xonsh(xenv, inp, run=True, mode="exec"):
     check_xonsh_ast(xenv, inp, run=run, mode=mode)
 
 
+def eval_code(inp, mode="eval", **loc_vars):
+    obs = PARSER.parse(inp, debug_level=1)
+    bytecode = compile(obs, "<test-xonsh-ast>", mode)
+    return eval(bytecode, loc_vars)
+
+
 #
 # Tests
 #
@@ -162,7 +167,7 @@ def test_fstring_adaptor(inp, exp):
     assert isinstance(joined_str_node, ast.JoinedStr)
     node = ast.Expression(body=joined_str_node)
     code = compile(node, "<test_fstring_adaptor>", mode="eval")
-    builtins.__xonsh__.env = {"HOME": "/foo/bar", "FOO": "HO", "BAR": "ME"}
+    XSH.env = {"HOME": "/foo/bar", "FOO": "HO", "BAR": "ME"}
     obs = eval(code)
     assert exp == obs
 
@@ -390,6 +395,39 @@ def test_if_else_expr_expr():
     check_ast("42+5 if 1 == 2 else 65-5")
 
 
+def test_subscription_syntaxes():
+    assert eval_code("[1, 2, 3][-1]") == 3
+    assert eval_code("[1, 2, 3][-1]") == 3
+    assert eval_code("'string'[-1]") == "g"
+
+
+@pytest.fixture
+def arr_container():
+    # like numpy.r_
+    class Arr:
+        def __getitem__(self, item):
+            return item
+
+    return Arr()
+
+
+def test_subscription_special_syntaxes(arr_container):
+    assert eval_code("arr[1, 2, 3]", arr=arr_container) == (1, 2, 3)
+    # dataframe
+    assert eval_code('arr[["a", "b"]]', arr=arr_container) == ["a", "b"]
+
+
+# todo: enable this test
+@pytest.mark.xfail
+def test_subscription_special_syntaxes_2(arr_container):
+    # aliases
+    d = {}
+    eval_code("d[arr.__name__]=True", arr=arr_container, d=d)
+    assert d == {"Arr": True}
+    # extslice
+    assert eval_code('arr[:, "2"]') == 2
+
+
 def test_str_idx():
     check_ast('"hello"[0]')
 
@@ -1041,6 +1079,11 @@ def test_lambda_x_star_y_kwargs():
     check_ast("lambda x, *, y, **kwargs: 42")
 
 
+@skip_if_pre_3_8
+def test_lambda_x_divide_y_star_z_kwargs():
+    check_ast("lambda x, /, y, *, z, **kwargs: 42")
+
+
 def test_call_range():
     check_ast("range(6)")
 
@@ -1209,12 +1252,12 @@ def test_rshift_op_three():
     check_ast("42 >> 65 >> 1 >> 7")
 
 
-@skip_if_no_walrus
+@skip_if_pre_3_8
 def test_named_expr():
     check_ast("(x := 42)")
 
 
-@skip_if_no_walrus
+@skip_if_pre_3_8
 def test_named_expr_list():
     check_ast("[x := 42, x + 1, x + 2]")
 
@@ -1368,6 +1411,10 @@ def test_equals_annotation():
     check_stmts("x : int = 42")
 
 
+def test_equals_annotation_empty():
+    check_stmts("x : int")
+
+
 def test_dict_keys():
     check_stmts('x = {"x": 1}\nx.keys()')
 
@@ -1560,6 +1607,11 @@ def test_yield_x_y():
     check_stmts("yield x, y", False)
 
 
+@skip_if_pre_3_8
+def test_return_x_starexpr():
+    check_stmts("yield x, *[y, z]", False)
+
+
 def test_yield_from_x():
     check_stmts("yield from x", False)
 
@@ -1580,6 +1632,11 @@ def test_return_x_y():
     check_stmts("return x, y", False)
 
 
+@skip_if_pre_3_8
+def test_return_x_starexpr():
+    check_stmts("return x, *[y, z]", False)
+
+
 def test_if_true():
     check_stmts("if True:\n  pass")
 
@@ -1927,6 +1984,16 @@ def test_func_x_star_y_kwargs():
     check_stmts("def f(x, *, y, **kwargs):\n  return 42")
 
 
+@skip_if_pre_3_8
+def test_func_x_divide():
+    check_stmts("def f(x, /):\n  return 42")
+
+
+@skip_if_pre_3_8
+def test_func_x_divide_y_star_z_kwargs():
+    check_stmts("def f(x, /, y, *, z, **kwargs):\n  return 42")
+
+
 def test_func_tx():
     check_stmts("def f(x:int):\n  return x")
 
@@ -2033,22 +2100,22 @@ def test_async_await():
     check_stmts("async def f():\n    await fut\n", False)
 
 
-@skip_if_no_walrus
+@skip_if_pre_3_8
 def test_named_expr_args():
     check_stmts("id(x := 42)")
 
 
-@skip_if_no_walrus
+@skip_if_pre_3_8
 def test_named_expr_if():
     check_stmts("if (x := 42) > 0:\n  x += 1")
 
 
-@skip_if_no_walrus
+@skip_if_pre_3_8
 def test_named_expr_elif():
     check_stmts("if False:\n  pass\nelif x := 42:\n  x += 1")
 
 
-@skip_if_no_walrus
+@skip_if_pre_3_8
 def test_named_expr_while():
     check_stmts("y = 42\nwhile (x := y) < 43:\n  y += 1")
 
@@ -2113,8 +2180,16 @@ def test_dollar_sub():
     check_xonsh_ast({}, "$(ls)", False)
 
 
-def test_dollar_sub_space():
-    check_xonsh_ast({}, "$(ls )", False)
+@pytest.mark.parametrize(
+    "expr",
+    [
+        "$(ls )",
+        "$( ls)",
+        "$( ls )",
+    ],
+)
+def test_dollar_sub_space(expr):
+    check_xonsh_ast({}, expr, False)
 
 
 def test_ls_dot():
@@ -2179,8 +2254,16 @@ def test_bang_sub():
     check_xonsh_ast({}, "!(ls)", False)
 
 
-def test_bang_sub_space():
-    check_xonsh_ast({}, "!(ls )", False)
+@pytest.mark.parametrize(
+    "expr",
+    [
+        "!(ls )",
+        "!( ls)",
+        "!( ls )",
+    ],
+)
+def test_bang_sub_space(expr):
+    check_xonsh_ast({}, expr, False)
 
 
 def test_bang_ls_dot():
@@ -2215,6 +2298,10 @@ def test_bang_ls_envvar_listval():
     check_xonsh_ast({"WAKKA": [".", "."]}, "!(ls $WAKKA)", False)
 
 
+def test_bang_envvar_args():
+    check_xonsh_ast({"LS": "ls"}, "!($LS .)", False)
+
+
 def test_question():
     check_xonsh_ast({}, "range?")
 
@@ -2476,6 +2563,10 @@ def test_ls_quotes_3_space():
     check_xonsh_ast({}, '$[ls "wakka jawaka baraka"]', False)
 
 
+def test_leading_envvar_assignment():
+    check_xonsh_ast({}, "![$FOO='foo' $BAR=2 echo r'$BAR']", False)
+
+
 def test_echo_comma():
     check_xonsh_ast({}, "![echo ,]", False)
 
@@ -3112,3 +3203,45 @@ def test_syntax_error_augassign_cmp(exp):
 def test_syntax_error_bar_kwonlyargs():
     with pytest.raises(SyntaxError):
         PARSER.parse("def spam(*):\n   pass\n", mode="exec")
+
+
+@skip_if_pre_3_8
+def test_syntax_error_bar_posonlyargs():
+    with pytest.raises(SyntaxError):
+        PARSER.parse("def spam(/):\n   pass\n", mode="exec")
+
+
+@skip_if_pre_3_8
+def test_syntax_error_bar_posonlyargs_no_comma():
+    with pytest.raises(SyntaxError):
+        PARSER.parse("def spam(x /, y):\n   pass\n", mode="exec")
+
+
+def test_syntax_error_nondefault_follows_default():
+    with pytest.raises(SyntaxError):
+        PARSER.parse("def spam(x=1, y):\n   pass\n", mode="exec")
+
+
+@skip_if_pre_3_8
+def test_syntax_error_posonly_nondefault_follows_default():
+    with pytest.raises(SyntaxError):
+        PARSER.parse("def spam(x, y=1, /, z):\n   pass\n", mode="exec")
+
+
+def test_syntax_error_lambda_nondefault_follows_default():
+    with pytest.raises(SyntaxError):
+        PARSER.parse("lambda x=1, y: x", mode="exec")
+
+
+@skip_if_pre_3_8
+def test_syntax_error_lambda_posonly_nondefault_follows_default():
+    with pytest.raises(SyntaxError):
+        PARSER.parse("lambda x, y=1, /, z: x", mode="exec")
+
+
+def test_get_repo_url():
+    PARSER.parse(
+        "def get_repo_url():\n"
+        "    raw = $(git remote get-url --push origin).rstrip()\n"
+        "    return raw.replace('https://github.com/', '')\n"
+    )
