@@ -10,7 +10,10 @@ import warnings
 import pytest
 
 from xonsh import __version__
-from xonsh.platform import ON_WINDOWS
+from xonsh.platform import (
+    ON_WINDOWS,
+    HAS_PYGMENTS,
+)
 from xonsh.lexer import Lexer
 
 from xonsh.tools import (
@@ -29,7 +32,6 @@ from xonsh.tools import (
     env_path_to_str,
     escape_windows_cmd_string,
     executables_in,
-    expand_case_matching,
     expand_path,
     find_next_break,
     is_bool,
@@ -52,6 +54,7 @@ from xonsh.tools import (
     to_bool,
     to_bool_or_int,
     to_bool_or_none,
+    to_int_or_none,
     to_dynamic_cwd_tuple,
     to_logfile_opt,
     pathsep_to_set,
@@ -62,7 +65,6 @@ from xonsh.tools import (
     is_nonstring_seq_of_strings,
     pathsep_to_upper_seq,
     seq_to_upper_pathsep,
-    expandvars,
     is_int_as_str,
     is_slice_as_str,
     ensure_timestamp,
@@ -77,8 +79,17 @@ from xonsh.tools import (
     deprecated,
     is_writable_file,
     balanced_parens,
+    ends_with_colon_token,
     iglobpath,
     all_permutations,
+    register_custom_style,
+    simple_random_choice,
+    is_completion_mode,
+    to_completion_mode,
+    is_completions_display_value,
+    to_completions_display_value,
+    expand_case_matching,
+    expandvars,
 )
 from xonsh.environ import Env
 
@@ -94,6 +105,15 @@ ENCODE_ENV_ONLY = {"XONSH_ENCODING_ERRORS": "strict"}
 PATHEXT_ENV = {"PATHEXT": [".COM", ".EXE", ".BAT"]}
 
 
+def test_random_choice():
+    lst = [1, 2, 3]
+    r = simple_random_choice(lst)
+    assert r in lst
+
+    with pytest.raises(ValueError):
+        simple_random_choice(range(1010101))
+
+
 def test_subproc_toks_x():
     exp = "![x]"
     obs = subproc_toks("x", lexer=LEXER, returnline=True)
@@ -614,6 +634,29 @@ def test_balanced_parens(line, exp):
         assert not obs
 
 
+@pytest.mark.parametrize(
+    "line, exp",
+    [
+        ("if 1:", True),
+        ("elif 2: #comment", True),
+        ("elif 3: #colon comment:", True),
+        ("else: ", True),
+        ("for s in '#not-a-comment':", True),
+        ("", False),
+        ("#comment", False),
+        ("#colon comment:", False),
+        ("print('hello')", False),
+        ("print('hello') #colon comment:", False),
+    ],
+)
+def test_ends_with_colon_token(line, exp):
+    obs = ends_with_colon_token(line, lexer=LEXER)
+    if exp:
+        assert obs
+    else:
+        assert not obs
+
+
 @pytest.mark.parametrize(
     "line, mincol, exp",
     [
@@ -702,6 +745,11 @@ def test_always_true(inp):
     assert always_true(inp)
 
 
+@pytest.mark.parametrize("inp,exp", [(42, 42), ("42", 42), ("None", None)])
+def test_to_optional_int(inp, exp):
+    assert to_int_or_none(inp) == exp
+
+
 @pytest.mark.parametrize("inp", [42, "42"])
 def test_always_false(inp):
     assert not always_false(inp)
@@ -910,8 +958,8 @@ def expand(path):
         (b"~/../", "~/../"),
     ],
 )
-def test_env_path_getitem(inp, exp, xonsh_builtins, env):
-    xonsh_builtins.__xonsh__.env = env
+def test_env_path_getitem(inp, exp, xession, env):
+    xession.env = env
     obs = EnvPath(inp)[0]  # call to __getitem__
     if env.get("EXPAND_ENV_VARS"):
         assert expand(exp) == obs
@@ -933,9 +981,9 @@ def test_env_path_getitem(inp, exp, xonsh_builtins, env):
         ),
     ],
 )
-def test_env_path_multipath(inp, exp, xonsh_builtins, env):
+def test_env_path_multipath(inp, exp, xession, env):
     # cases that involve path-separated strings
-    xonsh_builtins.__xonsh__.env = env
+    xession.env = env
     if env == TOOLS_ENV:
         obs = [i for i in EnvPath(inp)]
         assert [expand(i) for i in exp] == obs
@@ -958,8 +1006,8 @@ def test_env_path_multipath(inp, exp, xonsh_builtins, env):
         (["/home/wakka", pathlib.Path("~/"), "~/"], ["/home/wakka", "~", "~/"]),
     ],
 )
-def test_env_path_with_pathlib_path_objects(inp, exp, xonsh_builtins):
-    xonsh_builtins.__xonsh__.env = TOOLS_ENV
+def test_env_path_with_pathlib_path_objects(inp, exp, xession):
+    xession.env = TOOLS_ENV
     # iterate over EnvPath to acquire all expanded paths
     obs = [i for i in EnvPath(inp)]
     assert [expand(i) for i in exp] == obs
@@ -1450,7 +1498,7 @@ def test_partial_string(leaders, prefix, quote):
     assert obs == exp
 
 
-def test_executables_in(xonsh_builtins):
+def test_executables_in(xession):
     expected = set()
     types = ("file", "directory", "brokensymlink")
     if ON_WINDOWS:
@@ -1483,7 +1531,7 @@ def test_executables_in(xonsh_builtins):
                 if executable and not _type == "brokensymlink":
                     os.chmod(path, stat.S_IXUSR | stat.S_IRUSR | stat.S_IWUSR)
             if ON_WINDOWS:
-                xonsh_builtins.__xonsh__.env = PATHEXT_ENV
+                xession.env = PATHEXT_ENV
                 result = set(executables_in(test_path))
             else:
                 result = set(executables_in(test_path))
@@ -1538,12 +1586,12 @@ def test_expand_case_matching(inp, exp):
         (b"${'unk'}${'foo'}bar", "${'unk'}barbar"),
     ],
 )
-def test_expandvars(inp, exp, xonsh_builtins):
+def test_expandvars(inp, exp, xession):
     """Tweaked for xonsh cases from CPython `test_genericpath.py`"""
     env = Env(
         {"foo": "bar", "spam": "eggs", "a_bool": True, "an_int": 42, "none": None}
     )
-    xonsh_builtins.__xonsh__.env = env
+    xession.env = env
     assert expandvars(inp) == exp
 
 
@@ -1565,8 +1613,8 @@ def test_expandvars(inp, exp, xonsh_builtins):
         ),
     ],
 )
-def test_ensure_timestamp(inp, fmt, exp, xonsh_builtins):
-    xonsh_builtins.__xonsh__.env["XONSH_DATETIME_FORMAT"] = "%Y-%m-%d %H:%M"
+def test_ensure_timestamp(inp, fmt, exp, xession):
+    xession.env["XONSH_DATETIME_FORMAT"] = "%Y-%m-%d %H:%M"
     obs = ensure_timestamp(inp, fmt)
     assert exp == obs
 
@@ -1583,14 +1631,14 @@ def test_ensure_timestamp(inp, fmt, exp, xonsh_builtins):
         ("~/$foo", False, "/$foo"),
     ],
 )
-def test_expand_path(expand_user, inp, expand_env_vars, exp_end, xonsh_builtins):
+def test_expand_path(expand_user, inp, expand_env_vars, exp_end, xession):
     if os.sep != "/":
         inp = inp.replace("/", os.sep)
         exp_end = exp_end.replace("/", os.sep)
 
     env = Env({"foo": "bar", "a_bool": True, "an_int": 42, "none": None})
     env["EXPAND_ENV_VARS"] = expand_env_vars
-    xonsh_builtins.__xonsh__.env = env
+    xession.env = env
 
     path = expand_path(inp, expand_user=expand_user)
 
@@ -1704,7 +1752,7 @@ def test_deprecated_past_expiry_raises_assertion_error(expired_version):
 
 
 @skip_if_on_windows
-def test_iglobpath_no_dotfiles(xonsh_builtins):
+def test_iglobpath_no_dotfiles(xession):
     d = os.path.dirname(__file__)
     g = d + "/*"
     files = list(iglobpath(g, include_dotfiles=False))
@@ -1712,7 +1760,7 @@ def test_iglobpath_no_dotfiles(xonsh_builtins):
 
 
 @skip_if_on_windows
-def test_iglobpath_dotfiles(xonsh_builtins):
+def test_iglobpath_dotfiles(xession):
     d = os.path.dirname(__file__)
     g = d + "/*"
     files = list(iglobpath(g, include_dotfiles=True))
@@ -1720,7 +1768,7 @@ def test_iglobpath_dotfiles(xonsh_builtins):
 
 
 @skip_if_on_windows
-def test_iglobpath_no_dotfiles_recursive(xonsh_builtins):
+def test_iglobpath_no_dotfiles_recursive(xession):
     d = os.path.dirname(__file__)
     g = d + "/**"
     files = list(iglobpath(g, include_dotfiles=False))
@@ -1728,14 +1776,14 @@ def test_iglobpath_no_dotfiles_recursive(xonsh_builtins):
 
 
 @skip_if_on_windows
-def test_iglobpath_dotfiles_recursive(xonsh_builtins):
+def test_iglobpath_dotfiles_recursive(xession):
     d = os.path.dirname(__file__)
     g = d + "/**"
     files = list(iglobpath(g, include_dotfiles=True))
     assert d + "/bin/.someotherdotfile" in files
 
 
-def test_iglobpath_empty_str(monkeypatch, xonsh_builtins):
+def test_iglobpath_empty_str(monkeypatch, xession):
     # makes sure that iglobpath works, even when os.scandir() and os.listdir()
     # fail to return valid results, like an empty filename
     def mockscandir(path):
@@ -1772,3 +1820,118 @@ def test_all_permutations():
         "ABC",
     }
     assert obs == exp
+
+
+@pytest.mark.parametrize(
+    "name, styles, refrules",
+    [
+        ("test1", {}, {}),  # empty styles
+        (
+            "test2",
+            {"Token.Literal.String.Single": "#ff0000"},
+            {"Token.Literal.String.Single": "#ff0000"},
+        ),  # str key
+        (
+            "test3",
+            {"Literal.String.Single": "#ff0000"},
+            {"Token.Literal.String.Single": "#ff0000"},
+        ),  # short str key
+        (
+            "test4",
+            {"RED": "#ff0000"},
+            {"Token.Color.RED": "#ff0000"},
+        ),  # color
+    ],
+)
+def test_register_custom_style(name, styles, refrules):
+    style = register_custom_style(name, styles)
+    if HAS_PYGMENTS:
+        assert style is not None
+
+        for rule, color in style.styles.items():
+            if str(rule) in refrules:
+                assert refrules[str(rule)] == color
+
+
+@pytest.mark.parametrize(
+    "val, exp",
+    [
+        ("default", True),
+        ("menu-complete", True),
+        ("def", False),
+        ("xonsh", False),
+        ("men", False),
+    ],
+)
+def test_is_completion_mode(val, exp):
+    assert is_completion_mode(val) is exp
+
+
+@pytest.mark.parametrize(
+    "val, exp",
+    [
+        ("", "default"),
+        (None, "default"),
+        ("default", "default"),
+        ("DEfaULT", "default"),
+        ("m", "menu-complete"),
+        ("mEnu_COMPlete", "menu-complete"),
+        ("menu-complete", "menu-complete"),
+    ],
+)
+def test_to_completion_mode(val, exp):
+    assert to_completion_mode(val) == exp
+
+
+@pytest.mark.parametrize(
+    "val",
+    [
+        "de",
+        "defa_ult",
+        "men_",
+        "menu_",
+    ],
+)
+def test_to_completion_mode_fail(val):
+    with pytest.warns(RuntimeWarning):
+        obs = to_completion_mode(val)
+        assert obs == "default"
+
+
+@pytest.mark.parametrize(
+    "val, exp",
+    [
+        ("none", True),
+        ("single", True),
+        ("multi", True),
+        ("", False),
+        (None, False),
+        ("argle", False),
+    ],
+)
+def test_is_completions_display_value(val, exp):
+    assert is_completions_display_value(val) == exp
+
+
+@pytest.mark.parametrize(
+    "val, exp",
+    [
+        ("none", "none"),
+        (False, "none"),
+        ("false", "none"),
+        ("single", "single"),
+        ("readline", "readline"),  # todo: check this
+        ("multi", "multi"),
+        (True, "multi"),
+        ("TRUE", "multi"),
+    ],
+)
+def test_to_completions_display_value(val, exp):
+    assert to_completions_display_value(val) == exp
+
+
+@pytest.mark.parametrize("val", [1, "", "argle"])
+def test_to_completions_display_value_fail(val):
+    with pytest.warns(RuntimeWarning):
+        obs = to_completions_display_value(val)
+        assert obs == "multi"
