@@ -4,7 +4,6 @@ from __future__ import unicode_literals, print_function
 import os
 import sys
 import ast
-import builtins
 import platform
 import subprocess
 import contextlib
@@ -13,6 +12,7 @@ from collections.abc import MutableMapping
 
 import pytest
 
+from xonsh.built_ins import XSH
 from xonsh.environ import Env
 from xonsh.base_shell import BaseShell
 
@@ -30,7 +30,6 @@ ON_AZURE_PIPELINES = os.environ.get("TF_BUILD", "") == "True"
 print("ON_AZURE_PIPELINES", repr(ON_AZURE_PIPELINES))
 print("os.environ['TF_BUILD']", repr(os.environ.get("TF_BUILD", "")))
 TEST_DIR = os.path.dirname(__file__)
-HAS_WALRUS = VER_FULL > (3, 8)
 
 # pytest skip decorators
 skip_if_on_conda = pytest.mark.skipif(
@@ -53,7 +52,7 @@ skip_if_on_darwin = pytest.mark.skipif(ON_DARWIN, reason="not Mac friendly")
 
 skip_if_on_travis = pytest.mark.skipif(ON_TRAVIS, reason="not Travis CI friendly")
 
-skip_if_no_walrus = pytest.mark.skipif(not HAS_WALRUS, reason="no assignment expr.")
+skip_if_pre_3_8 = pytest.mark.skipif(VER_FULL < (3, 8), reason="Python >= 3.8 feature")
 
 
 def sp(cmd):
@@ -85,14 +84,6 @@ class DummyShell:
         return self._shell
 
 
-class DummyCommandsCache:
-    def locate_binary(self, name):
-        return os.path.join(os.path.dirname(__file__), "bin", name)
-
-    def predict_threadable(self, cmd):
-        return True
-
-
 class DummyHistory:
 
     last_cmd_rtn = 0
@@ -107,7 +98,11 @@ class DummyHistory:
 
 class DummyEnv(MutableMapping):
 
-    DEFAULTS = {"XONSH_DEBUG": 1, "XONSH_COLOR_STYLE": "default"}
+    DEFAULTS = {
+        "XONSH_DEBUG": 1,
+        "XONSH_COLOR_STYLE": "default",
+        "VC_BRANCH_TIMEOUT": 1,
+    }
 
     def __init__(self, *args, **kwargs):
         self._d = self.DEFAULTS.copy()
@@ -166,12 +161,12 @@ class DummyEnv(MutableMapping):
 
 
 def check_exec(input, **kwargs):
-    builtins.__xonsh__.execer.exec(input, **kwargs)
+    XSH.execer.exec(input, **kwargs)
     return True
 
 
 def check_eval(input):
-    builtins.__xonsh__.env = Env(
+    XSH.env = Env(
         {
             "AUTO_CD": False,
             "XONSH_ENCODING": "utf-8",
@@ -180,13 +175,13 @@ def check_eval(input):
         }
     )
     if ON_WINDOWS:
-        builtins.__xonsh__.env["PATHEXT"] = [".COM", ".EXE", ".BAT", ".CMD"]
-    builtins.__xonsh__.execer.eval(input)
+        XSH.env["PATHEXT"] = [".COM", ".EXE", ".BAT", ".CMD"]
+    XSH.execer.eval(input)
     return True
 
 
 def check_parse(input):
-    tree = builtins.__xonsh__.execer.parse(input, ctx=None)
+    tree = XSH.execer.parse(input, ctx=None)
     return tree
 
 
@@ -202,23 +197,43 @@ def nodes_equal(x, y):
         type(y),
     )
     if isinstance(x, (ast.Expr, ast.FunctionDef, ast.ClassDef)):
-        assert x.lineno == y.lineno, (
-            "Ast nodes do not have the same line number : %s != %s"
-            % (x.lineno, y.lineno,)
+        assert (
+            x.lineno == y.lineno
+        ), "Ast nodes do not have the same line number : %s != %s" % (
+            x.lineno,
+            y.lineno,
         )
-        assert x.col_offset == y.col_offset, (
-            "Ast nodes do not have the same column offset number : %s != %s"
-            % (x.col_offset, y.col_offset)
+        assert (
+            x.col_offset == y.col_offset
+        ), "Ast nodes do not have the same column offset number : %s != %s" % (
+            x.col_offset,
+            y.col_offset,
         )
     for (xname, xval), (yname, yval) in zip(ast.iter_fields(x), ast.iter_fields(y)):
-        assert xname == yname, (
-            "Ast nodes fields differ : %s (of type %s) != %s (of type %s)"
-            % (xname, type(xval), yname, type(yval))
+        assert (
+            xname == yname
+        ), "Ast nodes fields differ : %s (of type %s) != %s (of type %s)" % (
+            xname,
+            type(xval),
+            yname,
+            type(yval),
         )
-        assert type(xval) == type(yval), (
-            "Ast nodes fields differ : %s (of type %s) != %s (of type %s)"
-            % (xname, type(xval), yname, type(yval))
+        assert type(xval) == type(
+            yval
+        ), "Ast nodes fields differ : %s (of type %s) != %s (of type %s)" % (
+            xname,
+            type(xval),
+            yname,
+            type(yval),
         )
     for xchild, ychild in zip(ast.iter_child_nodes(x), ast.iter_child_nodes(y)):
         assert nodes_equal(xchild, ychild), "Ast node children differs"
     return True
+
+
+def completions_from_result(results):
+    if isinstance(results, tuple):
+        results, lprefix = results
+    if results is None:
+        return set()
+    return results
